--[[
# Element: Phasing Indicator

Toggles the visibility of an indicator based on the unit's phasing relative to the player.

## Widget

PhasedIndicator - Any UI widget.

## Notes

A default texture will be applied if the widget is a Texture and doesn't have a texture or a color set.

## Examples

    -- Position and size
    local PhasedIndicator = self:CreateTexture(nil, 'OVERLAY')
    PhasedIndicator:SetSize(16, 16)
    PhasedIndicator:SetPoint('TOPLEFT', self)

    -- Register it with oUF
    self.PhasedIndicator = PhasedIndicator
--]]

local _, ns = ...
local oUF = ns.oUF

local function Update(self, event)
	local element = self.PhasedIndicator

	--[[ Callback: PhasedIndicator:PreUpdate()
	Called before the element has been updated.

	* self - the PhasedIndicator element
	--]]
	if(element.PreUpdate) then
		element:PreUpdate()
	end

	local isInSamePhase = UnitInPhase(self.unit)
	if(isInSamePhase) then
		element:Hide()
	else
		element:Show()
	end

	--[[ Callback: PhasedIndicator:PostUpdate(isInSamePhase)
	Called after the element has been updated.

	* self          - the PhasedIndicator element
	* isInSamePhase - indicates whether the element is hidden (boolean)
	--]]
	if(element.PostUpdate) then
		return element:PostUpdate(isInSamePhase)
	end
end

local function Path(self, ...)
	--[[ Override: PhasedIndicator.Override(self, event, ...)
	Used to completely override the internal update function.

	* self  - the parent object
	* event - the event triggering the update (string)
	* ...   - the arguments accompanying the event
	--]]
	return (self.PhasedIndicator.Override or Update) (self, ...)
end

local function ForceUpdate(element)
	return Path(element.__owner, 'ForceUpdate')
end

local function Enable(self)
	local element = self.PhasedIndicator
	if(element) then
		element.__owner = self
		element.ForceUpdate = ForceUpdate

		self:RegisterEvent('UNIT_PHASE', Path, true)

		if(element:IsObjectType('Texture') and not element:GetTexture()) then
			element:SetTexture([[Interface\TargetingFrame\UI-PhasingIcon]])
		end

		return true
	end
end

local function Disable(self)
	local element = self.PhasedIndicator
	if(element) then
		element:Hide()

		self:UnregisterEvent('UNIT_PHASE', Path)
	end
end

oUF:AddElement('PhasedIndicator', Path, Enable, Disable)
