--[[
# Element: Raid Target Indicator

Handles the visibility and updating of an indicator based on the unit's raid target assignment.

## Widget

TargetMarkIndicator - A `Texture` used to display the raid target icon.

## Notes

A default texture will be applied if the widget is a Texture and doesn't have a texture set.

## Examples

    -- Position and size
    local TargetMarkIndicator = self:CreateTexture(nil, 'OVERLAY')
    TargetMarkIndicator:SetSize(16, 16)
    TargetMarkIndicator:SetPoint('TOPRIGHT', self)

    -- Register it with oUF
    self.TargetMarkIndicator = TargetMarkIndicator
--]]

local _, ns = ...
local oUF = ns.oUF
local RUF = RUF or LibStub("AceAddon-3.0"):GetAddon("RUF")

local GetRaidTargetIndex = GetRaidTargetIndex
local SetRaidTargetIconTexture = SetRaidTargetIconTexture

local function Update(self, event)
	local element = self.TargetMarkIndicator

	--[[ Callback: TargetMarkIndicator:PreUpdate()
	Called before the element has been updated.

	* self - the TargetMarkIndicator element
	--]]
	if(element.PreUpdate) then
		element:PreUpdate()
	end

	local index = GetRaidTargetIndex(self.unit)
	if(index) then
		SetRaidTargetIconTexture(element, index)
		element:Show()
	else
		element:Hide()
	end

	if RUF.db.global.TestMode == true then
		SetRaidTargetIconTexture(element, 1)
		element:Show()
	end

	--[[ Callback: TargetMarkIndicator:PostUpdate(index)
	Called after the element has been updated.

	* self  - the TargetMarkIndicator element
	* index - the index of the raid target marker (number?)[1-8]
	--]]
	if(element.PostUpdate) then
		return element:PostUpdate(index)
	end
end

local function Path(self, ...)
	--[[ Override: TargetMarkIndicator.Override(self, event)
	Used to completely override the internal update function.

	* self  - the parent object
	* event - the event triggering the update (string)
	--]]
	return (self.TargetMarkIndicator.Override or Update) (self, ...)
end

local function ForceUpdate(element)
	if(not element.__owner.unit) then return end
	return Path(element.__owner, 'ForceUpdate')
end

local function Enable(self)
	local element = self.TargetMarkIndicator
	if(element) then
		element.__owner = self
		element.ForceUpdate = ForceUpdate

		self:RegisterEvent('RAID_TARGET_UPDATE', Path, true)

		if(element:IsObjectType('Texture') and not element:GetTexture()) then
			element:SetTexture([[Interface\TargetingFrame\UI-RaidTargetingIcons]])
		end

		return true
	end
end

local function Disable(self)
	local element = self.TargetMarkIndicator
	if(element) then
		element:Hide()

		self:UnregisterEvent('RAID_TARGET_UPDATE', Path)
	end
end

oUF:AddElement('TargetMarkIndicator', Path, Enable, Disable)
